/* ympd
   (c) 2013-2014 Andrew Karpow <andy@ndyk.de>
   This project's homepage is: http://www.ympd.org
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

var socket;
var last_state;
var last_outputs;
var current_app;
var pagination = 0;
var browsepath;
var lastSongTitle = "";
var current_song = {};
var MAX_ELEMENTS_PER_PAGE = 500;
var isTouch = Modernizr.touch ? 1 : 0;
var connect_notify;

$.notifyDefaults({
  type: "info",
  newest_on_top: true,
  placement: { from: "bottom", align: "left" },
  animate :  { enter: 'animated fadeInUp',
               exit:  'animated fadeOutUp'
  },
  icon_type: 'class'
});

var app = $.sammy(function() {

  function runBrowse() {
    current_app = 'queue';
    $('#breadcrump').animate({'top': '34px'}, 300).fadeOut(100);
    //$('#salamisandwich').removeClass('hide').find("tr:gt(0)").remove();
    socket.send('MPD_API_GET_QUEUE,' + pagination);

    $('#panel-heading').html("<span class='glyphicon glyphicon-list'></span> Queue");
    $('#queue').addClass('active');
  }

  function prepare() {
    $('#nav_links > li').removeClass('active');
    $('#music-table-pager .page-btn').addClass('hide');
    $('#add-all-songs').hide();
    pagination = 0;
    browsepath = '';
  }

  this.get(/\#\/(\d+)/, function() {
    prepare();
    pagination = parseInt(this.params.splat[0]);
    runBrowse();
  });

  this.get(/\#\/browse\/(\d+)\/(.*)/, function() {
    prepare();
    browsepath = this.params.splat[1];
    pagination = parseInt(this.params.splat[0]);
    current_app = 'browse';
    $('#breadcrump').empty().append("<li><a href=\"#/browse/0/\">root</a></li>");
    if($('#breadcrump').is(':hidden')){
      $('#breadcrump').css('top', '30px').show().animate({'top':'0px'},300);
    }
    //$('#salamisandwich').removeClass('hide').find("tr:gt(0)").remove();
    socket.send('MPD_API_GET_BROWSE,' + pagination + ',' + (browsepath ? browsepath : "/"));
    // Don't add all songs from root
    if (browsepath) {
      var add_all_songs = $('#add-all-songs');
      add_all_songs.off(); // remove previous binds
      add_all_songs.on('click', function() {
        socket.send('MPD_API_ADD_TRACK,' + browsepath);
      });
      add_all_songs.show();
    }

    $('#panel-heading').html("<span class='glyphicon glyphicon-plus-sign'></span> Browse database: " + browsepath);
    var path_array = browsepath.split('/');
    var full_path = "";
    $.each(path_array, function(index, chunk) {
      if(path_array.length - 1 == index) {
        $('#breadcrump').append("<li class=\"active\">" + chunk + "</li>");
        return;
      }
      full_path = full_path + chunk;
      $('#breadcrump').append("<li><a href=\"#/browse/0/" + full_path + "\">" + chunk + "</a></li>");
      full_path += "/";
    });
    $('#browse').addClass('active');
  });

  this.get(/\#\/search\/(.*)/, function() {
    current_app = 'search';
    //$('#salamisandwich').find("tr:gt(0)").remove();
    var searchstr = this.params.splat[0];

    $('#search > div > input').val(searchstr);
    socket.send('MPD_API_SEARCH,' + searchstr);

    $('#panel-heading').text("Search: "+searchstr);
  });

  this.get("/", function(context) {
    context.redirect("#/0");
  });
});

function plugin_init() {
  $("#menu > li:gt(2)").remove();
  $("#nav_links > li:gt(4)").remove();

  $.getJSON( "/p/attach?t=" + Date.now(), function( data ) {
    for(i=0; i < data.length; i++){
      var menu = $.isEmptyObject(data[i].menu) ? "" : data[i].menu;
      var icon = $.isEmptyObject(data[i].icon) ? "" : data[i].icon;
      var path = $.isEmptyObject(data[i].path) ? "" : data[i].path;
      $("#menu").append(
        '<li role="separator" class="divider"></li>' +
        '<li><a data-icon="' + icon + '" href="#" onclick="event.preventDefault();activatePlugin(\'' +menu +'\',\'' +icon +'\',\'' +path +'\');return false;">' +
        ' ' +menu +'</a></li>'
      );
      $("#nav_links").append(
        '<li class="visible-xs menu-icon-xs"><a href="#" onclick="event.preventDefault();activatePlugin(\'' +menu +'\',\'' +icon +'\',\'' +path +'\');return false;">' +
        '<i class="material-icons">' +icon +'</i></a></li>'
      );
    }
  });

  $.getScript('/plugins/user.js');
}

$(document).ready(function() {
  webSocketConnect();

  $('#breadcrump').hide();

  $("#nav_links .volumeslider").slider(0);
  $("#nav_links .volumeslider").on('slider.newValue', function(evt, data) {
    $('#nav_links .progress-bar').text(data.val + "%");
    socket.send("MPD_API_SET_VOLUME," + data.val);
  });
  $('#carousel-example-generic .progressbar').slider(0);
  $('#carousel-example-generic .progressbar').on('slider.newValue', function(evt, data) {
    if (current_song && current_song.currentSongId >= 0) {
      var seekVal = Math.ceil(current_song.totalTime * (data.val / 100));
      socket.send("MPD_API_SET_SEEK," + current_song.currentSongId + "," + seekVal);
    }
  });

  $('#salamisandwich').hover(
    function() {
      $(this).addClass("visible-scrollbar");
    },
    function(){
      $(this).removeClass("visible-scrollbar");
    }
  );

  $('#addstream').on('shown.bs.modal', function () {
    $('#streamurl').focus();
  });
  $('#addstream form').on('submit', function (e) {
    addStream();
  });

  $("#nav_links .volume-menu").click(function(e) {
    e.stopPropagation();
  }); 

  $('#carousel-example-generic .cover-art').css("background-image", "url(/assets/no-image0.jpg)");

  plugin_init();

  /* filter */
  $("#title-filter").multifilter();

  touchDevice = (typeof window.ontouchstart) !== 'undefined';
  if (touchDevice)
    $("table.table-headerfixed tbody").css("padding-right", "12px");
});

function get_run_param() {
  var p = [];
  $('#plugin-form input.run').each(function(i) {
    if($(this).attr("type") == "radio" && $(this).prop('checked')){
      p[i] = $('#plugin-form [name=param]:checked').val();
    } else {
      p[i] = '"' + $(this).val() + '"';
    }
  });
  return p.join(' ');
}

function activatePlugin(menu, icon, path) {
  $("#activate-plugin")
    .data("menu", menu)
    .data("icon", icon)
    .data("path", path);

  /* output */
  $("#activate-plugin-title").text(" " + menu);
  $("#activate-plugin-icon")
    .removeClass()
    .addClass("material-icons")
    .text(icon);

  /* input form */
  $("#plugin-form-title").text(" " + menu);
  $("#plugin-form-msg").text("");
  $("#plugin-form-icon")
    .removeClass()
    .addClass("material-icons")
    .text(icon);

  $("#plugin-form .modal-body").load("/plugins/" +path +"/form.html?t=" +Date.now(), function(response, status, xhr){
    if (status == "error") {
      /* form.html not found, only run command */
      socket.send('ACTIVATE_PLUGIN,/home/pi/plugins/' +path);
    } else {

      /* smart form */
      $('#plugin-form input.run').each(function(i) {
        $(this).addClass("form-control").attr('spellcheck', false);
      });
      $('#plugin-form input.config').each(function(i) {
        $(this).addClass("form-control").attr('spellcheck', false);
        var query = {};
        query[$(this).attr('key')] = '';
        var target = "input.config[key='" + $(this).attr('key') + "']";
        $.get('/w' + $(this).attr('uri'), query,
          function(data){$(target).val(data);}, "html");
      });
      $("#plugin-form debug").append("<pre onclick='this.innerText=get_run_param();'></pre>");
      $("#plugin-form").modal();
    }
  });
}

function plugin_form_ok() {
  /* auto config update run */
  $('#plugin-form input.config').each(function(i) {
    var query = {};
    query[$(this).attr('key')] = $(this).val();
    $.get('/w' + $(this).attr('uri'), query);
  });

  $('#plugin-form').modal('hide');
  var param = get_run_param();
  if (param.length > 0) {
    var path = $("#activate-plugin").data('path');
    socket.send('ACTIVATE_PLUGIN,/home/pi/plugins/' + path + ',' + param);
  }
}

$(function() {
  var car = $('#carousel-example-generic');
  var hammer = new Hammer(car[0]);
  hammer.on('swipeleft',  function(){ car.carousel('next'); });
  hammer.on('swiperight', function(){ car.carousel('prev'); });

  car.on('slid.bs.carousel', function () {
    if ($("#carousel-1").hasClass("active") && $("#queue-icon").hasClass("glyphicon-list")) {
      toggleQueue();
      window.location.href = "#/";
    }
  });
});
 
$(window).load(function () {
  /* Table Height */
  var hsize = document.documentElement.clientHeight;
  $("#music-table").css("height", (hsize - 325) + "px");//340

  /* for iPhone SE */
  if (hsize < 580) {
    $("#player-buttonset")
    .css("margin-top", (hsize - 600)/2 + "px")
    .css("margin-top", "-50px")
    .css("margin-bottom", "30px")
    .css("opacity", "0.9");
  } else {
    $("#player-buttonset")
    .css("margin-top", "20px")
    .css("margin-bottom", "10px")
    .css("opacity", "1");
  }
});

$(window).resize(function() {
  /* Table Height */
  var hsize = document.documentElement.clientHeight;
  $("#music-table").css("height", (hsize - 325) + "px");//340

  /* for iPhone SE */
  if (hsize < 580) {
    $("#player-buttonset")
      .css("margin-top", (hsize - 600)/2 + "px")
      .css("margin-top", "-50px")
      .css("margin-bottom", "30px")
      .css("opacity", "0.9");
  } else {
    $("#player-buttonset")
      .css("margin-top", "20px")
      .css("margin-bottom", "10px")
      .css("opacity", "1");
  }
});

function toggleQueue() {
  var queue_icon   = $("#queue-icon");
  var queue_button = $("#queue-button");
  if (queue_icon.hasClass("glyphicon-plus-sign")) {
    queue_button.attr("href", "#/browse/0/");
    if (queue_button.attr("browse-path") != "")
      queue_button.attr("href", queue_button.attr("browse-path"));
    queue_icon.removeClass("glyphicon-plus-sign").addClass("glyphicon-list");
    $("#music-table").hide().fadeIn(180);
    $("#cover-btn-group").css('top', '145px').animate({'top': '110px'}, 300);
  } else {
    queue_button.attr("href", "#/");
    queue_button.attr("browse-path", location.hash);
    queue_icon.removeClass("glyphicon-list").addClass("glyphicon-plus-sign");
    $("#music-table").hide().fadeIn(400);
    $("#cover-btn-group").css('top', '110px').animate({'top': '145px'}, 300);
  }
}

function volume_plus(p) {
  var vol = parseInt($('#nav_links .volumeslider>div>div:first').text());
  var ret = vol+p >= 100 ? 100 : vol+p;
  $('#nav_links .progress-bar').text(ret + "%");
  return ret;
}

function volume_minus(m) {
  var vol = parseInt($('#nav_links .volumeslider>div>div:first').text());
  var ret = vol-m <= 0 ? 0 : vol-m;
  $('#nav_links .progress-bar').text(ret + "%");
  return ret;
}

function webSocketConnect() {
  if (typeof MozWebSocket != "undefined")
    socket = new MozWebSocket(get_appropriate_ws_url());
  else
    socket = new WebSocket(get_appropriate_ws_url());

  try {
    socket.onopen = function() {
      console.log("connected");
      connect_notify = $.notify("Connected to ympd");

      app.run();
      /* emit initial request for output names */
      socket.send("MPD_API_GET_OUTPUTS");
    };
    var html;
    socket.onmessage = function got_packet(msg) {
      var minutes;
      var seconds;
      if (msg.data === last_state || msg.data.length == 0)
        return;

      var obj = JSON.parse(msg.data.replace(/(\n)/g, '\\n'));

      switch (obj.type) {
        case "queue":
          if(current_app !== 'queue')
            break;

          $('#salamisandwich > tbody').empty();
          html = "";
          var html2 = "";
          var alb = "";
          var startval = 0;
          var endval = 0;
          //var rnd = $('#btnrandom').hasClass("active");
          for (var song in obj.data) {
            //if (!rnd) {
            if (alb !== obj.data[song].album) {
              if (alb !== '')
                html += outAlbumHeader(alb, startval, endval, html2);
              html2 = "";
              alb = obj.data[song].album;
              startval = endval;
            }
            //}
            minutes = Math.floor(obj.data[song].duration / 60);
            seconds = obj.data[song].duration - minutes * 60;
            html2 += "<tr sid=\"" + endval + "\" trackid=\"" + obj.data[song].id + "\"><td>" + (obj.data[song].pos + 1) + "</td>" +
              "<td>"+ obj.data[song].title +"</td>" +
              "<td class='hidden-xs'>"+ obj.data[song].artist +"</td>" +
              "<td class='hidden-xs'>"+ minutes + ":" + (seconds < 10 ? '0' : '') + seconds +
              "</td><td>"+
              "<a class=\"pull-right btn-group-hover text-center\" style=\"width:28px;\" href=\"#/\" " +
              "onclick=\"socket.send('MPD_API_RM_TRACK,' + $(this).parents('tr').attr('trackid')); $(this).parents('tr').remove();\">" +
              "<span class=\"glyphicon glyphicon-trash\"></span></a>" +
              "</td></tr>";
            endval++;
          }
          //if (!rnd) {
            if (alb !== "")
              html += outAlbumHeader(alb, startval, endval, html2);
          //}
          html += html2;
          $('#salamisandwich > tbody').append(html);

          if (obj.data.length > 0) {
            if(obj.data[obj.data.length-1].pos + 1 >= pagination + MAX_ELEMENTS_PER_PAGE)
              $('#next').removeClass('hide');
            if(pagination > 0)
              $('#prev').removeClass('hide');
          }

          $('#salamisandwich > tbody > tr').on( {
            click: function() {
              if (!$(this).hasClass("alb-text")) {
                $('#salamisandwich > tbody > tr').removeClass('active');
                socket.send('MPD_API_PLAY_TRACK,' + $(this).attr('trackid'));
                $(this).addClass('active');
              }
            }
          });
          break;
        case "dashboard":
          $.getJSON( "/RAM/.dashboard.json?t=" + Date.now(), function( data ) {
            var items = [];
            $.each( data, function(key, val) {
              if(key == "arm_freq" || key == "core_freq")
                return true;
              items.push("<tr><td>" + key + "</td><td>" + val + "</td></tr>");
            });
            $("#dashboard-contents").html(items.join(""));
            $("#canvas_holder").css("background-image", "url(/RAM/.latency.svg?t=" + Date.now() + ")").show();
            dashboard_meter(parseInt(data.arm_freq / 1000000), parseInt(data.core_freq / 1000000));
          });
          $('#wait-dashboard').hide();
          break;
        case "search":
          $('#wait').modal('hide');
          break;
        case "browse":
          if (current_app !== 'browse' && current_app !== 'search')
            break;

          /* The use of encodeURI() below might seem useless, but it's not. It prevents
           * some browsers, such as Safari, from changing the normalization form of the
           * URI from NFD to NFC, breaking our link with MPD.
           */
          $('#salamisandwich > tbody').empty();
          $("#music-table").hide().fadeIn(180);
          html = "";
          for (var item in obj.data) {
            switch(obj.data[item].type) {
              case "directory":
                html += "<tr uri=\"" + encodeURI(obj.data[item].dir) + "\" class=\"dir\">" +
                  "<td><span class=\"glyphicon glyphicon-folder-open\"></span></td>" +
                  "<td><a>" + basename(obj.data[item].dir) + "</a></td>" +
                  "<td class='hidden-xs'></td>" +
                  "<td class='hidden-xs'></td><td></td></tr>";
                break;
              case "playlist":
                html += "<tr uri=\"" + encodeURI(obj.data[item].plist) + "\" class=\"plist\">" +
                  "<td><span class=\"glyphicon glyphicon-list\"></span></td>" +
                  "<td><a>" + basename(obj.data[item].plist) + "</a></td>" +
                  "<td class='hidden-xs'></td>" + 
                  "<td class='hidden-xs'></td><td></td></tr>";
                break;
              case "song":
                minutes = Math.floor(obj.data[item].duration / 60);
                seconds = obj.data[item].duration - minutes * 60;

                html += "<tr uri=\"" + encodeURI(obj.data[item].uri) + "\" class=\"song\">" +
                  "<td><span class=\"glyphicon glyphicon-music\"></span></td>" +
                  "<td>"+ obj.data[item].title +"</td>" +
                  "<td class='hidden-xs'>"+ obj.data[item].artist +"</td>" + 
                  "<td class='hidden-xs'>"+ minutes + ":" + (seconds < 10 ? '0' : '') + seconds +
                  "</td><td></td></tr>";
                break;
              case "wrap":
                if(current_app == 'browse') {
                  $('#next').removeClass('hide');
                } else {
                  html += "<tr><td><span class=\"glyphicon glyphicon-remove\"></span></td>" + 
                    "<td>Too many results, please refine your search!</td>" + 
                    "<td class='hidden-xs'></td>" + 
                    "<td></td><td></td></tr>";
                }
                break;
            }

            if(pagination > 0)
              $('#prev').removeClass('hide');
          }
          $('#salamisandwich > tbody').append(html);

          function appendClickableIcon(appendTo, onClickAction, glyphicon) {
            $(appendTo).append(
              "<a role=\"button\" class=\"pull-right btn-group-hover text-center\" style=\"width:28px;\">" +
              "<span class=\"glyphicon glyphicon-" + glyphicon + "\"></span></a>")
              .find('a').click(function(e) {
                e.stopPropagation();
                socket.send(onClickAction + "," + decodeURI($(this).parents("tr").attr("uri")));
              $.notify($('td:nth-child(2)', $(this).parents("tr")).text() + " added");
            }).fadeTo('fast',1);
          }

          appendClickableIcon($("#salamisandwich > tbody > tr.dir > td:last-child"), 'MPD_API_ADD_TRACK', 'plus');
          appendClickableIcon($("#salamisandwich > tbody > tr.plist > td:last-child"), 'MPD_API_ADD_PLAYLIST', 'plus');
          appendClickableIcon($("#salamisandwich > tbody > tr.song > td:last-child"), 'MPD_API_ADD_TRACK', 'play');

          $('#salamisandwich > tbody > tr').on( {
            click: function() {
              switch($(this).attr('class')) {
                case 'dir':
                  pagination = 0;
                  browsepath = $(this).attr("uri");
                  $("#browse > a").attr("href", '#/browse/' + pagination + '/' + browsepath);
                  app.setLocation('#/browse/' + pagination+'/' + browsepath);
                  break;
                case 'song':
                  socket.send("MPD_API_ADD_TRACK," + decodeURI($(this).attr("uri")));
                  $.notify($('td:nth-child(2)', this).text() + " added");
                  break;
                case 'plist':
                  socket.send("MPD_API_ADD_PLAYLIST," + decodeURI($(this).attr("uri")));
                  $.notify("Playlist " + $('td:nth-child(2)', this).text() + " added");
                  break;
              }
            }
          });
          break;
        case "state":
          updatePlayIcon(obj.data.state);
          updateVolumeIcon(obj.data.volume);

          var progress = Math.floor(100 * obj.data.elapsedTime / obj.data.totalTime);
          $('#carousel-example-generic .progressbar').slider(progress);
          $('#nav_links .volumeslider').slider(obj.data.volume);
          $('#nav_links .progress-bar').text(obj.data.volume + "%");

          if(JSON.stringify(obj) === JSON.stringify(last_state))
            break;

          current_song.totalTime  = obj.data.totalTime;
          current_song.currentSongId = obj.data.currentsongid;
          var total_minutes = Math.floor(obj.data.totalTime / 60);
          var total_seconds = obj.data.totalTime - total_minutes * 60;

          var elapsed_minutes = Math.floor(obj.data.elapsedTime / 60);
          var elapsed_seconds = obj.data.elapsedTime - elapsed_minutes * 60;

          $('.counter')
            .text(elapsed_minutes + ":" + 
            (elapsed_seconds < 10 ? '0' : '') + elapsed_seconds + " / " +
            total_minutes + ":" + (total_seconds < 10 ? '0' : '') + total_seconds);

          $('#salamisandwich > tbody > tr').removeClass('active').css("font-weight", "");
          $('#salamisandwich > tbody > tr[trackid=' + obj.data.currentsongid + ']').addClass('active').css("font-weight", "bold");

          if (obj.data.random)
            $('#btnrandom').addClass("active");
          else
            $('#btnrandom').removeClass("active");

          if (obj.data.consume)
            $('.btnconsume').addClass("active");
          else
            $('.btnconsume').removeClass("active");

          if (obj.data.single)
            $('#btnsingle').addClass("active");
          else
            $('#btnsingle').removeClass("active");

          if (obj.data.crossfade)
            $('#btncrossfade').addClass("active");
          else
            $('#btncrossfade').removeClass("active");

          if(obj.data.repeat)
            $('#btnrepeat').addClass("active");
          else
            $('#btnrepeat').removeClass("active");

          last_state = obj;
          break;
        case "outputnames":
          $('#btn-outputs-block button').remove();
          $.each(obj.data, function(id, name) {
            var btn = $('<button id="btnoutput' + id +
              '" class="btn btn-default" onclick="toggleoutput(this, ' + id +
              ')"><span class="glyphicon glyphicon-volume-up"></span> ' + name + '</button>');
            btn.appendTo($('#btn-outputs-block'));
          });
          /* remove cache, since the buttons have been recreated */
          last_outputs = '';
          break;
        case "outputs":
          if(JSON.stringify(obj) === JSON.stringify(last_outputs))
            break;
          $.each(obj.data, function(id, enabled) {
            if (enabled)
              $('#btnoutput' + id).addClass("active");
            else
              $('#btnoutput' + id).removeClass("active");
          });
          last_outputs = obj;
          break;
        case "disconnected":
          if (connect_notify != null) {
            connect_notify.update( {
              'type': "danger",
              'message': "ympd lost connection to MPD"
            });
          }
          break;
        case "update_queue":
          if(current_app === 'queue')
            socket.send('MPD_API_GET_QUEUE,'+pagination);
          break;
        case "song_change":
          $('#carousel-example-generic .album').text("");
          $('#carousel-example-generic .artist').text("");
          $('#carousel-example-generic .currenttrack').hide().text(" " + obj.data.title).fadeIn(500);
          var notification = "<strong><h4>" + obj.data.title + "</h4></strong>";

          if (obj.data.uri) {
            var default_img_no = Math.floor(Math.random() * 2);
            if (obj.data.uri.startsWith("http")) {
              var url = "url(/assets/no-image" + default_img_no + ".jpg)";
              $('#carousel-example-generic .cover-art').css("background-image", url);
            } else {
              cover_art_url(obj.data.artist, obj.data.album, "mega", function (err, lastfmurl) {
                var uri_raw = obj.data.uri.split("/");
                uri_raw.splice(-1, 1);
                $.each(uri_raw, function(i, val) {
                  uri_raw[i] = encodeURIComponent(val)
                    .replace(/\(/g, "\\(").replace(/\)/g, "\\)").replace(/\'/g, "\\'");
                });
                var uri     = "/" + uri_raw.join("/") + "/";
                var url     = "";
                url += "url(" + uri + "folder.jpg" + "), url(" + uri + "Folder.jpg" + "), ";
                if (typeof obj.data.artist === 'string' && typeof obj.data.album === 'string') {
                  url += "url(" + uri + encodeURIComponent(obj.data.artist).replace(/\(/g, "\\(").replace(/\)/g, "\\)").replace(/\'/g, "\\'") + "%20-%20" + encodeURIComponent(obj.data.album).replace(/\(/g, "\\(").replace(/\)/g, "\\)").replace(/\'/g, "\\'") + ".jpg" + "), ";
                }
                if (lastfmurl.length > 0)
                  url += "url(" + lastfmurl + "), ";
                url += "url(/assets/no-image"+ default_img_no  + ".jpg)";
                $('#carousel-example-generic .cover-art').css("background-image", url );
              });
            }
          }
          if(obj.data.album) {
            $('#carousel-example-generic .album').text(obj.data.album);
            notification += obj.data.album + "<br />";
          }
          if(obj.data.artist) {
            $('#carousel-example-generic .artist').text(obj.data.artist);
            notification += obj.data.artist + "<br />";
          }

          break;
        case "mpdhost":
          $('#mpdnas').val($.trim(obj.data.nas));
          $('#mpdnasuser').val($.trim(obj.data.nasuser));
          $('#mpdnaspw').val($.trim(obj.data.naspw));
          break;
        case "error":
          $.notify({
            message: obj.data
          }, {
            type: "danger",
          });
          $('#nav_links .setting-icon').removeClass('gly-spin');
          break;
        case "success":
          $.notify({
            message: obj.data
          }, {
            type: "success",
          });
          $('#nav_links .setting-icon').removeClass('gly-spin');
          break;
        case "nas_start":
          $.notify(obj.data);
          break;
        case "update_start":
          $(".modal-footer > button").prop("disabled", true);
          $("#update-progress .modal-body").text("");
          var checktimer = setInterval(function() {
            $("#update-progress").modal();
            $("#update-progress .modal-body").load("/RAM/.update.log?t=" + Date.now(),"",function(data, status, object) {
              if (data.endsWith("succeeded\n") || data.endsWith("completed\n")) {
                clearInterval(checktimer);
                $(".modal-footer > button").prop("disabled", false);
                $('#nav_links .setting-icon').removeClass('gly-spin');
              }
            });
          }, 200);
          break;
        case "plugin":
          plugin_init();
          if (obj.data == "stdout") {
            $("#activate-plugin .modal-body").html(obj.buf);
            $("#activate-plugin").modal();
          } else if(obj.data == "success") {
            $.notify({
              message: "success"
            }, {
              type: "success",
            });
          } else if(obj.data == "error") {
            $.notify({
              message: "error"
            }, {
              type: "danger",
            });
          }
          break;
        default:
          break;
      }
    };
    socket.onclose = function() {
      console.log("disconnected");
      connect_notify.update({
        'message': "Connection to ympd lost, retrying ",
        'type': "danger",
        'onClose': function(){ webSocketConnect();}
      });
    };
  } catch(exception) {
    alert('<p>Error' + exception + '</p>');
  }
}

function get_appropriate_ws_url()
{
  var pcol;
  var u = document.URL;

  /*
  /* We open the websocket encrypted if this page came on an
  /* https:// url itself, otherwise unencrypted
  /*/

  if (u.substring(0, 5) == "https") {
    pcol = "wss://";
    u = u.substr(8);
  } else {
    pcol = "ws://";
    if (u.substring(0, 4) == "http")
      u = u.substr(7);
  }

  u = u.split('#');

  return pcol + u[0] + "/ws";
}

var updateVolumeIcon = function(volume)
{
  $("#volume-icon")
  .removeClass("glyphicon-volume-off")
  .removeClass("glyphicon-volume-up")
  .removeClass("glyphicon-volume-down");

  if(volume == 0) {
    $("#volume-icon").addClass("glyphicon-volume-off");
  } else if (volume < 50) {
    $("#volume-icon").addClass("glyphicon-volume-down");
  } else {
    $("#volume-icon").addClass("glyphicon-volume-up");
  }
};

var updatePlayIcon = function(state)
{
  $("#carousel-example-generic .play-icon")
  .removeClass("glyphicon-play")
  .removeClass("glyphicon-pause");

  $('#track-icon')
  .removeClass("glyphicon-play")
  .removeClass("glyphicon-pause")
  .removeClass("glyphicon-stop");

  if(state == 1) { // stop
    $(".glyphicon-headphones").removeClass("gly-head");
    $("#carousel-example-generic .play-icon").addClass("glyphicon-play");
    $('#track-icon').addClass("glyphicon-stop");
    socket.send('MPD_API_SET_STOP');
  } else if(state == 2) { // pause
    $(".glyphicon-headphones").addClass("gly-head");
    $("#carousel-example-generic .play-icon").addClass("glyphicon-pause");
    $('#track-icon').addClass("glyphicon-play");
  } else { // play
    $(".glyphicon-headphones").removeClass("gly-head");
    $("#carousel-example-generic .play-icon").addClass("glyphicon-play");
    $('#track-icon').addClass("glyphicon-pause");
  }
};

function systemReboot() {
  $("#reboot-confirm").modal('hide');
  socket.send('SYSTEM_REBOOT');
  $.notify("symphonic-mpd reboot... ");
}

function systemPowerOff() {
  $("#poweroff-confirm").modal('hide');
  socket.send('SYSTEM_POWEROFF');
  $.notify("symphonic-mpd shutdown... ");
}

function updateDB() {
  socket.send('MPD_API_UPDATE_DB');
  $.notify("Updating MPD Database... ");
}

function clickPlay() {
  if ($('#track-icon').hasClass('glyphicon-play')) {
    socket.send('MPD_API_SET_PAUSE');
  } else {
    socket.send('MPD_API_SET_PLAY');
  }
}

function basename(path) {
  return path.split('/').reverse()[0];
}

$('#btnrandom').on('click', function (e) {
  socket.send("MPD_API_TOGGLE_RANDOM," + ($(this).hasClass('active') ? 0 : 1));
});
$('#btnconsume').on('click', function (e) {
  socket.send("MPD_API_TOGGLE_CONSUME," + ($(this).hasClass('active') ? 0 : 1));
});
$('#btnsingle').on('click', function (e) {
  socket.send("MPD_API_TOGGLE_SINGLE," + ($(this).hasClass('active') ? 0 : 1));
});
$('#btncrossfade').on('click', function(e) {
  socket.send("MPD_API_TOGGLE_CROSSFADE," + ($(this).hasClass('active') ? 0 : 1));
});
$('#btnrepeat').on('click', function (e) {
  socket.send("MPD_API_TOGGLE_REPEAT," + ($(this).hasClass('active') ? 0 : 1));
});

function toggleoutput(button, id) {
  socket.send("MPD_API_TOGGLE_OUTPUT," + id + "," + ($(button).hasClass('active') ? 0 : 1));
}

$('#btnnotify').on('click', function (e) {
  if ($.cookie("notification") === "true") {
    $.cookie("notification", false);
  } else {
    Notification.requestPermission(function (permission) {
      if (!('permission' in Notification))
        Notification.permission = permission;

      if (permission === "granted") {
        $.cookie("notification", true, { expires: 424242 });
        $('btnnotify').addClass("active");
      }
    });
  }
});

function getDashboard() {
  $("#canvas_holder").hide();
  $("#canvas_holder").css("background-image", "");
  $('#wait-dashboard').show();
  socket.send('MPD_API_GET_DASHBOARD');
}

function inplaceUpdate() {
  $("#update_button").prop("disabled", true);
  socket.send('INPLACE_UPDATE');
  $("#update_icon").css('color', 'Black');
  $("#version_available").text("");
  $('#settings').modal('hide');
  $('#nav_links .setting-icon').addClass('gly-spin');
}

function getHost() {
  socket.send('MPD_API_GET_MPDHOST');

  function onEnter(event) {
    if (event.which == 13)
      confirmSettings();
  }

  $('#mpdnas').keypress(onEnter);
  $('#mpdnasuser').keypress(onEnter);
  $('#mpdnaspw').keypress(onEnter);

  var version_available = {};

  var version="v0.9.x";
  $.getJSON( "http://mpd.sytes.net/release/" + version + "/.smpd-version.json?t=" + Date.now(), function( data ) {
    version_available.major = data.major;
    version_available.minor = data.minor;
    version_available.revision = data.revision;

    $.getJSON( "/.version.json?t=" + Date.now(), function( data ) {
      $("#version_applied").text(" v" + data.major + "." + data.minor + "." + data.revision );
      if (version_available.major == data.major && version_available.minor == data.minor && version_available.revision > data.revision ) {
        $("#update_button").prop("disabled", false);
        $("#version_available").text(' v' + version_available.major + '.' + version_available.minor + '.' + version_available.revision + ' available');
        $("#update_icon").css('color', 'Red');
      }
    });
  });
}

$('#search').submit(function () {
  app.setLocation("#/search/" + $('#search > div > input').val());
  $('#wait').modal('show');
  setTimeout(function() {
    $('#wait').modal('hide');
  }, 10000);
  return false;
});

$('#music-table-pager .page-btn').on('click', function (e) {
  switch ($(this).text()) {
    case "Next":
      pagination += MAX_ELEMENTS_PER_PAGE;
      break;
    case "Previous":
      pagination -= MAX_ELEMENTS_PER_PAGE;
      if(pagination <= 0)
        pagination = 0;
      break;
  }

  switch(current_app) {
    case "queue":
      app.setLocation('#/' + pagination);
      break;
    case "browse":
      app.setLocation('#/browse/' + pagination+'/' + browsepath);
      break;
  }
  e.preventDefault();
});

function addStream() {
  if ($('#streamurl').val().length > 0)
    socket.send('MPD_API_ADD_TRACK,' + $('#streamurl').val());
  $('#streamurl').val("");
  $('#addstream').modal('hide');
}

function saveQueue() {
  if ($('#playlistname').val().length > 0)
    socket.send('MPD_API_SAVE_QUEUE,' + $('#playlistname').val());
  $('#savequeue').modal('hide');
}

function confirmSettings() {
  $('#settings').modal('hide');
  $('#nav_links .setting-icon').addClass('gly-spin');

  $(".modal-footer > button").prop("disabled", true);
  $("#nas-progress .modal-body").text("");
  $("#nas-progress").modal();
  var checktimer = setInterval(function() {
    $("#nas-progress .modal-body").load("/RAM/.nas_setting.log?t=" + Date.now(),"",function(data, status, object) {
      if (data.endsWith("succeeded\n")) {
        clearInterval(checktimer);
        $(".modal-footer > button").prop("disabled", false);
        $('#nav_links .setting-icon').removeClass('gly-spin');
        $.notify({
          message: "NAS Setting complete :)"
        }, {
          type: "success"
        });
      }
    });
  }, 200);

  var send_str = 'MPD_API_SET_MPDHOST,' + $('#mpdnas').val() + ',' + $('#mpdnasuser').val();
  if ($.trim($('#mpdnaspw').val()).length > 0) {
    send_str += ',' + $('#mpdnaspw').val();
  }
  socket.send(send_str);
}

$('#mpd_password_set > button').on('click', function (e) {
  socket.send('MPD_API_SET_MPDPASS,');
  $('#mpd_pw').val("");
  $('#mpd_pw_con').val("");
  $('#mpd_password_set').addClass('hide');
});

function notificationsSupported() {
  return "Notification" in window;
}

$(document).keydown(function(e) {
  if (e.target.tagName == 'INPUT')
    return;
  switch (e.which) {
    case 37: //left
      socket.send('MPD_API_SET_PREV');
      break;
    case 39: //right
      socket.send('MPD_API_SET_NEXT');
      break;
    case 32: //space
      clickPlay();
      break;
    case 67: //consume
      socket.send('MPD_API_TOGGLE_RANDOM,' + ($('#btnconsume').hasClass('active') ? 0 : 1));
      break;
    case 82: //repeat
      socket.send('MPD_API_TOGGLE_REPEAT,' + ($('#btnrepeat').hasClass('active') ? 0 : 1));
      break;
    case 83: //shuffle
      socket.send('MPD_API_TOGGLE_RANDOM,' + ($('#btnrandom').hasClass('active') ? 0 : 1));
      break;
    default:
      return;
  }
  e.preventDefault();
});

function dashboard_meter(_arm_freq, _core_freq){
  var arm_freq  = _arm_freq;
  var core_freq = _core_freq;
  var arm_ctx = document.getElementById('arm').getContext('2d');
  var myChart = new Chart(arm_ctx, {
    type: 'doughnut',
    data: {
      datasets: [
        {
          borderColor: ["white", "white", "white", "white", "white", "white", "white", "white", "white", "white", "white", "white", "white", "white", "white", "rgba(0, 0, 0, 0)"],
          data: [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 5],
          backgroundColor: ["gray", "gray", "gray", "gray", "gray", "gray", "gray", "gray", "gray", "gray", "gray", "gray", "red", "red", "red", "rgba(255,255,255,0)"]
        },
        {
          borderColor: ["#eaeaea", "rgba(0,0,0,0)", "rgba(0,0,0,0)"],
          data: [270 * arm_freq / 1500, 270 * (1500 - arm_freq) / 1500, 90],
          backgroundColor: ["rgba(153,255,51,0.6)", "rgba(0,0,0,0)", "rgba(0,0,0,0)"]
        }
      ]
    },
    options: {
      tooltips: {enabled: false}, title: {display: true, text: ["ARM", arm_freq], padding: -74}, rotation: 0.75 * Math.PI, animation: {animateScale: true}
    }
  });
  myChart = null;
  var core_ctx = document.getElementById('core').getContext('2d');
  var myChart2 = new Chart(core_ctx, {
    type: 'doughnut',
    data: {
      datasets: [
        {
          borderColor: ["white", "white", "white", "white", "white", "white", "rgba(0,0,0,0)"],
          data: [1, 1, 1, 1, 1, 1, 2],
          backgroundColor: ["gray", "gray", "gray", "gray", "red", "red", "rgba(0,0,0,0)"]
        },
        {
          borderColor: ["#eaeaea", "rgba(0,0,0,0)", "rgba(0,0,0,0)"],
          data: [270 * core_freq / 600, 270 * (600 - core_freq) / 600, 90],
          backgroundColor: ["rgba(153,255,51,0.6)", "rgba(0,0,0,0)", "rgba(0,0,0,0)"]
        }
      ]
    },
    options: {tooltips: {enabled: false}, title: {display: true, text: ["CORE", core_freq], padding: -74}, rotation: 0.75 * Math.PI, animation: {animateScale: true}}
  });
  myChart2 = null;
}

function outAlbumHeader(alb, st, en, htm) {
  var idval = st + '-' + en;
  var html = "<tr class=\"alb-text\" >"+ "<td>" +
    "<a onclick=\"albumToggle('" + idval + "')\">" +
    "<span id=\"" + idval + "\" class=\"glyphicon glyphicon-circle-arrow-up\"></span>" +
    "</a>" + "</td>" + 
    "<td>「" + alb + "」</td>" +
    "<td class='hidden-xs'>" + "</td>" +
    "<td class='hidden-xs'>" +
    "</td><td>" +
    "<a class=\"pull-right btn-group-hover text-center\" style=\"width:28px;\" href=\"#/\" " +
    "onclick=\"socket.send('MPD_API_RM_RANGE,"+ st +"," + en + "'); socket.send('MPD_API_GET_QUEUE,0');\">" +
    "<span class=\"glyphicon glyphicon-trash\"></span></a>" +
    "</td></tr>" +
    htm;
  return html;
}

function albumToggle(idv) {
  var i;
  var sp = $('#' + idv);
  var spary = idv.split("-");
  if(sp.hasClass("glyphicon-circle-arrow-up")) {
    sp.removeClass("glyphicon-circle-arrow-up");
    sp.addClass("glyphicon-circle-arrow-down");
    for (i = spary[0]; i < spary[1]; i++) {
      $('tr[sid=' + i + ']').hide();
    }
  } else {
    sp.removeClass("glyphicon-circle-arrow-down");
    sp.addClass("glyphicon-circle-arrow-up");
    for(i = spary[0]; i < spary[1]; i++) {
      $('tr[sid=' + i + ']').show();
    }
  }
  return false;
}
